@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 """
-Copyright (C) 2009,2010 Wolfgang Rohdewald <wolfgang@rohdewald.de>
+Copyright (C) 2009-2011 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
 kajongg is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 """
 
 import socket, subprocess, time, datetime, os
@@ -28,12 +28,10 @@ from twisted.python.failure import Failure
 from twisted.internet.address import UNIXAddress
 from PyQt4.QtCore import SIGNAL, SLOT, Qt, QTimer, QObject
 from PyQt4.QtGui import QDialog, QDialogButtonBox, QVBoxLayout, QGridLayout, \
-    QLabel, QComboBox, QLineEdit, QPushButton, \
+    QLabel, QComboBox, QLineEdit, QPushButton, QFormLayout, \
     QProgressBar, QRadioButton, QSpacerItem, QSizePolicy
 
-from PyKDE4.kdecore import KUser
-from PyKDE4.kdeui import KDialogButtonBox
-from PyKDE4.kdeui import KMessageBox
+from kde import KMessageBox, KDialogButtonBox, KUser
 
 from util import m18n, m18nc, logWarning, logException, socketName, english, \
     appdataDir, logInfo
@@ -44,12 +42,12 @@ from common import InternalParameters, PREF
 from game import Players
 from query import Transaction, Query
 from board import Board
-from client import Client, Client1
+from client import Client1
 from statesaver import StateSaver
 from meld import Meld
 
 from guiutil import ListComboBox
-from scoringengine import Ruleset, PredefinedRuleset
+from scoringengine import Ruleset
 
 class LoginDialog(QDialog):
     """login dialog for server"""
@@ -85,32 +83,19 @@ class LoginDialog(QDialog):
         self.connect(buttonBox, SIGNAL("accepted()"), self, SLOT("accept()"))
         self.connect(buttonBox, SIGNAL("rejected()"), self, SLOT("reject()"))
         vbox = QVBoxLayout(self)
-        grid = QGridLayout()
-        lblServer = QLabel(m18n('Game server:'))
-        grid.addWidget(lblServer, 0, 0)
+        self.grid = QFormLayout()
         self.cbServer = QComboBox()
         self.cbServer.setEditable(True)
-        grid.addWidget(self.cbServer, 0, 1)
-        lblServer.setBuddy(self.cbServer)
-        lblUsername = QLabel(m18n('Username:'))
-        grid.addWidget(lblUsername, 1, 0)
+        self.grid.addRow(m18n('Game server:'), self.cbServer)
         self.cbUser = QComboBox()
         self.cbUser.setEditable(True)
-        self.cbUser.setMinimumWidth(350) # is this good for all platforms?
-        lblUsername.setBuddy(self.cbUser)
-        grid.addWidget(self.cbUser, 1, 1)
-        self.lblPassword = QLabel(m18n('Password:'))
-        grid.addWidget(self.lblPassword, 2, 0)
+        self.grid.addRow(m18n('Username:'), self.cbUser)
         self.edPassword = QLineEdit()
         self.edPassword.setEchoMode(QLineEdit.PasswordEchoOnEdit)
-        grid.addWidget(self.edPassword, 2, 1)
-        self.lblPassword.setBuddy(self.edPassword)
-        self.lblRuleset = QLabel(m18nc('kajongg', 'Ruleset:'))
-        grid.addWidget(self.lblRuleset, 3, 0)
+        self.grid.addRow(m18n('Password:'), self.edPassword)
         self.cbRuleset = ListComboBox()
-        grid.addWidget(self.cbRuleset, 3, 1)
-        self.lblRuleset.setBuddy(self.cbRuleset)
-        vbox.addLayout(grid)
+        self.grid.addRow(m18nc('kajongg', 'Ruleset:'), self.cbRuleset)
+        vbox.addLayout(self.grid)
         vbox.addWidget(buttonBox)
         pol = QSizePolicy()
         pol.setHorizontalPolicy(QSizePolicy.Expanding)
@@ -120,16 +105,19 @@ class LoginDialog(QDialog):
         """user entered OK"""
         if self.host == 'localhost':
             # we have localhost if we play a Local Game: client and server are identical,
-            # we have no security concerns about  creating a new account
+            # we have no security concerns about creating a new account
             Players.createIfUnknown(unicode(self.cbUser.currentText()))
         QDialog.accept(self)
 
     def serverChanged(self, dummyText=None):
         """the user selected a different server"""
+        records = Query('select player.name from player, passwords '
+                'where passwords.url=? and passwords.player = player.id', list([self.host])).records
         self.cbUser.clear()
-        self.cbUser.addItems(list(x[1] for x in self.servers if x[0] == self.host))
+        self.cbUser.addItems(list(x[0] for x in records))
         if not self.cbUser.count():
-            self.cbUser.addItem(KUser(os.geteuid()).fullName())
+            user = KUser() if os.name == 'nt' else KUser(os.geteuid())
+            self.cbUser.addItem(user.fullName() or user.loginName())
         hostName = self.host
         userNames = [x[1] for x in self.servers if x[0] == hostName]
         if userNames:
@@ -137,9 +125,9 @@ class LoginDialog(QDialog):
             if userIdx >= 0:
                 self.cbUser.setCurrentIndex(userIdx)
         showPW = self.host != Query.localServerName
-        self.lblPassword.setVisible(showPW)
+        self.grid.labelForField(self.edPassword).setVisible(showPW)
         self.edPassword.setVisible(showPW)
-        self.lblRuleset.setVisible(not showPW)
+        self.grid.labelForField(self.cbRuleset).setVisible(not showPW)
         self.cbRuleset.setVisible(not showPW)
         if not showPW:
             self.cbRuleset.clear()
@@ -213,30 +201,18 @@ class AddUserDialog(QDialog):
         self.connect(self.buttonBox, SIGNAL("accepted()"), self, SLOT("accept()"))
         self.connect(self.buttonBox, SIGNAL("rejected()"), self, SLOT("reject()"))
         vbox = QVBoxLayout(self)
-        grid = QGridLayout()
-        lblServer = QLabel(m18n('Game server:'))
-        grid.addWidget(lblServer, 0, 0)
+        grid = QFormLayout()
         self.cbServer = QComboBox()
         self.cbServer.setEditable(True)
-        grid.addWidget(self.cbServer, 0, 1)
-        lblServer.setBuddy(self.cbServer)
-        lblUsername = QLabel(m18n('Username:'))
-        grid.addWidget(lblUsername, 1, 0)
+        grid.addRow(m18n('Game server:'), self.cbServer)
         self.edUser = QLineEdit()
-        lblUsername.setBuddy(self.edUser)
-        grid.addWidget(self.edUser, 1, 1)
-        self.lblPassword = QLabel(m18n('Password:'))
-        grid.addWidget(self.lblPassword, 2, 0)
+        grid.addRow(m18n('Username:'), self.edUser)
         self.edPassword = QLineEdit()
         self.edPassword.setEchoMode(QLineEdit.PasswordEchoOnEdit)
-        grid.addWidget(self.edPassword, 2, 1)
-        self.lblPassword.setBuddy(self.edPassword)
-        self.lblPassword2 = QLabel(m18n('Repeat password:'))
-        grid.addWidget(self.lblPassword2, 3, 0)
+        grid.addRow(m18n('Password:'), self.edPassword)
         self.edPassword2 = QLineEdit()
         self.edPassword2.setEchoMode(QLineEdit.PasswordEchoOnEdit)
-        grid.addWidget(self.edPassword2, 3,  1)
-        self.lblPassword2.setBuddy(self.edPassword2)
+        grid.addRow(m18n('Repeat password:'), self.edPassword2)
         vbox.addLayout(grid)
         vbox.addWidget(self.buttonBox)
         pol = QSizePolicy()
@@ -356,7 +332,7 @@ class SelectChow(QDialog):
             QDialog.keyPressEvent(self, event)
 
 class SelectKong(QDialog):
-    """asks which of the possible chows is wanted"""
+    """asks which of the possible kongs is wanted"""
     def __init__(self, kongs):
         QDialog.__init__(self)
         self.setWindowTitle('Kajongg')
@@ -472,7 +448,7 @@ class ClientDialog(QDialog):
         else:
             msecs = 50
             self.progressBar.setMinimum(0)
-            self.progressBar.setMaximum(self.client.game.ruleset.claimTimeout * 1000 / msecs)
+            self.progressBar.setMaximum(self.client.game.ruleset.claimTimeout * 1000 // msecs)
             self.progressBar.reset()
             self.timer.start(msecs)
 
@@ -486,9 +462,9 @@ class ClientDialog(QDialog):
         vertical = view.width() > view.height() * 1.2
         if vertical:
             height = (len(self.buttons) + 1) * btnHeight * 1.2
-            width = (cwi.width() - cwi.height() ) / 2
+            width = (cwi.width() - cwi.height() ) // 2
             geometry.setX(cwi.width() - width)
-            geometry.setY(min(cwi.height()/3,  cwi.height() - height))
+            geometry.setY(min(cwi.height()//3, cwi.height() - height))
         else:
             handBoard = self.client.game.myself.handBoard
             if not handBoard:
@@ -498,7 +474,7 @@ class ClientDialog(QDialog):
             hbRightBottom = view.mapFromScene(handBoard.mapToScene(handBoard.rect().bottomRight()))
             width = hbRightBottom.x() - hbLeftTop.x()
             height = btnHeight
-            geometry.setY(cwi.height()  - height)
+            geometry.setY(cwi.height() - height)
             geometry.setX(hbLeftTop.x())
         for idx, btn in enumerate(self.buttons + [self.progressBar]):
             self.layout.addWidget(btn, idx+1 if vertical else 0, idx+1 if not vertical else 0)
@@ -581,8 +557,8 @@ class HumanClient(Client1):
     """a human client"""
     # pylint: disable=R0904
     # disable warning about too many public methods
-    serverProcess = None
-    socketServerProcess = None
+    # pylint: disable=R0902
+    # we have 11 instance attributes, more than pylint likes
 
     def __init__(self, tableList, callback):
         Client1.__init__(self)
@@ -600,28 +576,26 @@ class HumanClient(Client1):
         self.useSocket = self.loginDialog.host == Query.localServerName
         if self.useSocket or self.loginDialog.host == 'localhost':
             if not self.serverListening():
+                if os.name == 'nt':
+                    port = HumanClient.findFreePort()
+                    common.PREF.serverPort = port
+                else:
+                    port = None
+                self.startLocalServer(port)
                 # give the server up to 5 seconds time to start
-                HumanClient.startLocalServer(self.useSocket)
                 for loop in range(50):
                     if self.serverListening():
                         break
                     time.sleep(0.1)
         self.username = self.loginDialog.username
         self.ruleset = self.__defineRuleset()
+        self.__msg = None # helper for delayed error messages
         self.login(callback)
 
     def __defineRuleset(self):
         """find out what ruleset to use"""
         if InternalParameters.autoPlayRuleset:
-            for ruleset in PredefinedRuleset.rulesets():
-                if ruleset.name == InternalParameters.autoPlayRuleset:
-                    return ruleset
-            try:
-                return Ruleset(InternalParameters.autoPlayRuleset)
-            except Exception as exception:
-                InternalParameters.autoPlay = False
-                InternalParameters.autoPlayRuleset = False
-                raise exception
+            return InternalParameters.autoPlayRuleset
         elif InternalParameters.autoPlay:
             return Ruleset.selectableRulesets()[0]
         else:
@@ -644,10 +618,23 @@ class HumanClient(Client1):
         """True if we are talking to a Local Game Server"""
         return self.useSocket
 
+    @staticmethod
+    def findFreePort():
+        """find an unused port on the current system.
+        used when we want to start a local server on windows"""
+        for port in range(2000, 9000):
+            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+            sock.settimeout(1)
+            try:
+                sock.connect(('127.0.0.1', port))
+            except socket.error:
+                return port
+        logException('cannot find a free port')
+
     def serverListening(self):
         """is somebody listening on that port?"""
-        if self.useSocket:
-            sock = socket.socket(socket.AF_UNIX,  socket.SOCK_STREAM)
+        if self.useSocket and os.name != 'nt':
+            sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
             sock.settimeout(1)
             try:
                 sock.connect(socketName())
@@ -659,7 +646,7 @@ class HumanClient(Client1):
             else:
                 return True
         else:
-            sock = socket.socket(socket.AF_INET,  socket.SOCK_STREAM)
+            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
             sock.settimeout(1)
             try:
                 sock.connect((self.loginDialog.host, self.loginDialog.port))
@@ -668,50 +655,33 @@ class HumanClient(Client1):
             else:
                 return True
 
-    @staticmethod
-    def startLocalServer(useSocket):
+    def startLocalServer(self, port):
         """start a local server"""
         try:
-            cmd = './kajonggserver.py'
-            if not os.path.exists(cmd):
-                cmd = 'kajonggserver'
-            args = [cmd]
+            if os.path.exists('kajonggserver.py'):
+                args = ['python', 'kajonggserver.py']
+            else:
+                args = ['kajonggserver']
             if InternalParameters.showTraffic:
                 args.append('--showtraffic')
             if InternalParameters.showSql:
                 args.append('--showsql')
-            if useSocket:
-                args.append('--socket')
+            if self.useSocket or os.name == 'nt':
+                args.append('--local')
+            if port:
+                args.append('--port=%d' % port)
+            if self.useSocket:
                 args.append('--db=%slocal.db' % appdataDir())
-            process = subprocess.Popen(args)
+            process = subprocess.Popen(args, shell=os.name=='nt')
             logInfo(m18n('started the local kajongg server: pid=<numid>%1</numid> %2',
                 process.pid, ' '.join(args)))
-            if useSocket:
-                HumanClient.socketServerProcess = process
-            else:
-                HumanClient.serverProcess = process
         except OSError, exc:
             logException(exc)
 
-    @staticmethod
-    def stopLocalServers():
-        """stop the local servers we started"""
-        for process in [HumanClient.serverProcess, HumanClient.socketServerProcess]:
-            if process:
-                logInfo(m18n('stopped the local kajongg server: pid=<numid>%1</numid>',
-                    process.pid))
-                process.terminate()
-        HumanClient.serverProcess = None
-        HumanClient.socketServerProcess = None
-
-    def __del__(self):
-        """if we go away and we started a local server, stop it again"""
-        HumanClient.stopLocalServers()
-
     def remote_tablesChanged(self, tables):
         """update table list"""
-        Client.remote_tablesChanged(self, tables)
-        self.tableList.load(self.tables)
+        Client1.remote_tablesChanged(self, tables)
+        self.tableList.loadTables(self.tables)
 
     def readyForGameStart(self, tableid, gameid, seed, playerNames, shouldSave=True):
         """playerNames are in wind order ESWN"""
@@ -724,10 +694,19 @@ class HumanClient(Client1):
                 "If you answer with NO, you will be removed from the table.")
             wantStart = KMessageBox.questionYesNo (None, msg) == KMessageBox.Yes
         if wantStart:
-            Client.readyForGameStart(self, tableid, gameid, seed, playerNames, shouldSave=shouldSave)
+            self.__msg = Client1.readyForGameStart(self, tableid, gameid, seed, playerNames, shouldSave=shouldSave)
+            if self.__msg:
+                # if we call KMessageBox directly here, the twisted reactor somehow gets out of sync.
+                # it will try to parse the last atomic item again and fail because that is not a list. Happens
+                # with twisted 10.2 and twisted 11.0
+                QTimer.singleShot(0, self.warnData)
         else:
             self.answers.append(Message.NO)
 
+    def warnData(self):
+        """see comment above about calling KMessageBox directly"""
+        logWarning(m18n(*self.__msg))
+
     def readyForHandStart(self, playerNames, rotateWinds):
         """playerNames are in wind order ESWN"""
         if InternalParameters.field:
@@ -745,13 +724,13 @@ class HumanClient(Client1):
 
     def clientReadyForHandStart(self, dummy, playerNames, rotateWinds):
         """callback, called after the client player said yes, I am ready"""
-        Client.readyForHandStart(self, playerNames, rotateWinds)
+        Client1.readyForHandStart(self, playerNames, rotateWinds)
 
     def ask(self, move, answers, callback=None):
         """server sends move. We ask the user. answers is a list with possible answers,
         the default answer being the first in the list."""
         if not InternalParameters.field:
-            return Client.ask(self, move, answers, callback)
+            return Client1.ask(self, move, answers, callback)
         deferred = Deferred()
         if callback:
             deferred.addCallback(callback)
@@ -772,7 +751,7 @@ class HumanClient(Client1):
     def selectChow(self, chows):
         """which possible chow do we want to expose?"""
         if self.game.autoPlay:
-            return Client.selectChow(self, chows)
+            return Client1.selectChow(self, chows)
         if len(chows) == 1:
             return chows[0]
         selDlg = SelectChow(chows)
@@ -782,7 +761,7 @@ class HumanClient(Client1):
     def selectKong(self, kongs):
         """which possible kong do we want to declare?"""
         if self.game.autoPlay:
-            return Client.selectKong(self, kongs)
+            return Client1.selectKong(self, kongs)
         if len(kongs) == 1:
             return kongs[0]
         selDlg = SelectKong(kongs)
@@ -793,7 +772,7 @@ class HumanClient(Client1):
         """the user answered our question concerning move"""
         if self.game.autoPlay:
             self.game.hidePopups()
-            return Client.ask(self, move, answers)
+            return Client1.ask(self, move, answers)
         myself = self.game.myself
         if answer == Message.Discard:
             # do not remove tile from hand here, the server will tell all players
@@ -807,7 +786,7 @@ class HumanClient(Client1):
 
     def answerError(self, answer, move, answers):
         """an error happened while determining the answer to server"""
-        logException('%s %s %s %s' % (self.game.myself.name, answer,  move,  answers))
+        logException('%s %s %s %s' % (self.game.myself.name, answer, move, answers))
 
     def remote_abort(self, tableid, message, *args):
         """the server aborted this game"""
@@ -832,7 +811,7 @@ class HumanClient(Client1):
                 if self.game.autoPlay and not InternalParameters.field:
                     gameWinner = max(self.game.players, key=lambda x: x.balance)
                     writer = csv.writer(open('kajongg.csv','a'), delimiter=';')
-                    row = [str(self.game.seed)]
+                    row = [InternalParameters.AI, str(self.game.seed)]
                     for player in sorted(self.game.players, key=lambda x: x.name):
                         row.append(player.name)
                         row.append(player.balance)
@@ -865,7 +844,6 @@ class HumanClient(Client1):
         if isinstance(result, Failure):
             logException(result)
         InternalParameters.reactor.stop()
-        HumanClient.stopLocalServers()
         # we may be in a Deferred callback generated in abortGame which would
         # catch sys.exit as an exception
         # and the qt4reactor does not quit the app when being stopped
@@ -894,7 +872,7 @@ class HumanClient(Client1):
         or adduser/deluser/change passwd encoded in the username"""
         factory = pb.PBClientFactory()
         reactor = InternalParameters.reactor
-        if self.useSocket:
+        if self.useSocket and os.name != 'nt':
             self.connector = reactor.connectUNIX(socketName(), factory)
         else:
             self.connector = reactor.connectTCP(self.loginDialog.host, self.loginDialog.port, factory)
@@ -904,7 +882,7 @@ class HumanClient(Client1):
         return factory.login(cred, client=self)
 
     def adduser(self, host, name, passwd, callback, callbackParameter):
-        """create  a user account"""
+        """create a user account"""
         assert host is not None
         if self.loginDialog.host != Query.localServerName:
             adduserDialog = AddUserDialog()
@@ -920,7 +898,7 @@ class HumanClient(Client1):
                 raise Exception(m18n('Aborted creating a user account'))
             name, passwd = adduserDialog.username, adduserDialog.password
         self.loginDialog.password = passwd
-        adduserCmd =  SERVERMARK.join(['adduser', name, passwd])
+        adduserCmd = SERVERMARK.join(['adduser', name, passwd])
         self.loginCommand(adduserCmd).addCallback(callback,
             callbackParameter).addErrback(self._loginFailed, callbackParameter)
 
@@ -928,7 +906,7 @@ class HumanClient(Client1):
         """login failed"""
         message = failure.getErrorMessage()
         dlg = self.loginDialog
-        host, name,  passwd = dlg.host, dlg.username, dlg.password
+        host, name, passwd = dlg.host, dlg.username, dlg.password
         if 'Wrong username' in message:
             msg = m18nc('USER is not known on SERVER',
                 '%1 is not known on %2, do you want to open an account?', name, host)
@@ -937,7 +915,12 @@ class HumanClient(Client1):
                 self.adduser(host, name, passwd, self.adduserOK, callback)
                 return
         else:
-            logWarning(message)
+            if self.useSocket and os.name != 'nt':
+                connectMsg = m18n('calling kajongg server on UNIX socket %1', socketName())
+            else:
+                connectMsg = m18n('calling kajongg server on %1:<numid>%2</numid>',
+                    self.loginDialog.host, self.loginDialog.port)
+            logWarning(connectMsg + ': ' + message)
         if callback:
             callback()
 
@@ -971,7 +954,7 @@ class HumanClient(Client1):
                         list([self.loginDialog.password, host, playerId]))
                 else:
                     Query('insert into passwords(url,player,password) values(?,?,?)',
-                        list([host,  playerId, self.loginDialog.password]))
+                        list([host, playerId, self.loginDialog.password]))
         self.perspective = perspective
         if callback:
             callback()
