@@ -201,11 +201,11 @@ KWin4App::KWin4App(QWidget *parent)
 KWin4App::~KWin4App()
 {
   kDebug() << "~KWin4App()";
-  if (mDoc) delete mDoc;
-  if (mView) delete mView;
-  if (mScene) delete mScene;
-  if (mTheme) delete mTheme;
-  if (mMyChatDlg) delete mMyChatDlg;
+  delete mDoc;
+  delete mView;
+  delete mScene;
+  delete mTheme;
+  delete mMyChatDlg;
   kDebug() << "~KWin4App()";
 }
 
@@ -323,20 +323,20 @@ void KWin4App::initGUI()
   KStandardGameAction::load(this, SLOT(menuOpenGame()), actionCollection());
   KStandardGameAction::save(this, SLOT(menuSaveGame()), actionCollection());
   action = KStandardGameAction::end(this, SLOT(endGame()), actionCollection());
-  action->setWhatsThis(i18n("Aborts a currently played game. No winner will be declared."));
+  action->setWhatsThis(i18n("Ends a currently played game. No winner will be declared."));
   KStandardGameAction::hint(this, SLOT(askForHint()), actionCollection());
   KStandardGameAction::quit(this, SLOT(close()), actionCollection());
   
-  action = actionCollection()->addAction("network_conf");
+  action = actionCollection()->addAction( QLatin1String( "network_conf" ));
   action->setText(i18n("&Network Configuration..."));
   connect(action, SIGNAL(triggered(bool) ), SLOT(configureNetwork()));
 
-  action = actionCollection()->addAction("network_chat");
+  action = actionCollection()->addAction( QLatin1String( "network_chat" ));
   action->setText(i18n("Network Chat..."));
   connect(action, SIGNAL(triggered(bool) ), SLOT(configureChat()));
 
-  action = actionCollection()->addAction("statistics");
-  action->setIcon(KIcon("view-statistics"));
+  action = actionCollection()->addAction( QLatin1String( "statistics" ));
+  action->setIcon(KIcon( QLatin1String( "view-statistics" )));
   action->setText(i18n("&Show Statistics"));
   connect(action, SIGNAL(triggered(bool)), SLOT(showStatistics()));
   action->setToolTip(i18n("Show statistics."));
@@ -351,8 +351,8 @@ void KWin4App::initGUI()
   QStringList themes(mThemeFiles.keys());
   themes.sort();
 
-  action = actionCollection()->addAction("theme", new KSelectAction(i18n("Theme"), this));
-  action->setIcon(KIcon("games-config-theme"));
+  action = actionCollection()->addAction( QLatin1String( "theme" ) , new KSelectAction(i18n("Theme" ), this));
+  action->setIcon(KIcon( QLatin1String( "games-config-theme" )));
   ((KSelectAction*)action)->setItems(themes);
   connect( action, SIGNAL(triggered(int)), SLOT(changeTheme(int)) );
   kDebug() << "Setting current theme item to" << mThemeIndexNo;
@@ -361,7 +361,7 @@ void KWin4App::initGUI()
   // Debug
   if (global_debug>0)
   {
-    action = actionCollection()->addAction("file_debug");
+    action = actionCollection()->addAction( QLatin1String( "file_debug" ));
     action->setText(i18n("Debug KGame"));
     connect(action, SIGNAL(triggered(bool) ), SLOT(debugKGame()));
   }
@@ -769,7 +769,7 @@ void KWin4App::slotGameOver(int status, KPlayer* p, KGame* /*me*/)
   else if (status==2) // Abort
   {
     EndGame(TBrk);
-    QString  m=i18n(" Game aborted. Please restart next round.");
+    QString  m=i18n(" Game ended. Please restart next round.");
     displayStatusMessage(m);
   }
   else
