@@ -23,7 +23,15 @@
 
 #include "kgrcanvas.h"
 #include "kgrselector.h"
-#include "kgrsoundbank.h"
+
+#ifdef ENABLE_SOUND_SUPPORT
+    #ifdef KGOLDRUNNER_USE_OPENAL
+    #include "kgrsounds.h"
+    #else
+    #include "kgrsoundbank.h"
+    #endif
+#endif
+
 #include "kgreditor.h"
 #include "kgrlevelplayer.h"
 #include "kgrdialog.h"
@@ -92,6 +100,8 @@ KGrGame::KGrGame (KGrCanvas * theView,
         programFreeze (false),
         effects       (0),
         fx            (NumSounds),
+        soundOn       (false),
+        stepsOn       (false),
         editor        (0)
 {
     dbgLevel = 0;
@@ -361,7 +371,8 @@ void KGrGame::settings (const int action)
     bool onOff = false;
     switch (action) {
     case PLAY_SOUNDS:
-        toggleSoundsOnOff();
+    case PLAY_STEPS:
+        toggleSoundsOnOff (action);
         break;
     case STARTUP_DEMO:
         // Toggle the startup demo to be on or off.
@@ -406,6 +417,14 @@ void KGrGame::setInitialTheme (const QString & themeFilepath)
 
 void KGrGame::initGame()
 {
+#ifndef ENABLE_SOUND_SUPPORT
+    KGrMessage::information (view, i18n ("No Sound"),
+        i18n ("Warning: This copy of KGoldrunner has no sound.\n"
+              "\n"
+              "This is because no development versions of the OpenAL and "
+              "SndFile libraries were present when it was compiled and built."),
+              "WarningNoSound");
+#endif
     kDebug() << "Entered, draw the initial graphics now ...";
 
     // Get the most recent collection and level that was played by this user.
@@ -452,13 +471,17 @@ void KGrGame::initGame()
 
 #ifdef ENABLE_SOUND_SUPPORT
     // Set up sounds, if required in config.
-    bool soundOnOff = gameGroup.readEntry ("Sound", false);
-    kDebug() << "Sound" << soundOnOff;
-    if (soundOnOff) {
+    soundOn = gameGroup.readEntry ("Sound", false);
+    kDebug() << "Sound" << soundOn;
+    if (soundOn) {
         loadSounds();
         effects->setMuted (false);
     }
-    emit setToggle ("options_sounds", soundOnOff);
+    emit setToggle ("options_sounds", soundOn);
+
+    stepsOn = gameGroup.readEntry ("StepSounds", false);
+    kDebug() << "StepSounds" << stepsOn;
+    emit setToggle ("options_steps", stepsOn);
 #endif
 
     dbk1 << "Owner" << gameList.at (gameIndex)->owner
@@ -883,12 +906,16 @@ void KGrGame::playSound (const int n, const bool onOff)
     }
     static int fallToken = -1;
     if (onOff) {
-	int token = effects->play (fx [n]);
+	int token = -1;
+        if (stepsOn || ((n != StepSound) && (n != ClimbSound))) {
+            token = effects->play (fx [n]);
+        }
         if (n == FallSound) {
             fallToken = token;
         }
     }
-    else {
+    // Only the falling sound can get individually turned off.
+    else if ((n == FallSound) && (fallToken >= 0)) {
 	effects->stop (fallToken);
 	fallToken = -1;
     }
@@ -898,6 +925,13 @@ void KGrGame::playSound (const int n, const bool onOff)
 void KGrGame::endLevel (const int result)
 {
     dbk << "Return to KGrGame, result:" << result;
+
+#ifdef ENABLE_SOUND_SUPPORT
+    if (effects) {			// If sounds have been loaded, cut off
+        effects->stopAllSounds();	// all sounds that are in progress.
+    }
+#endif
+
     if (! levelPlayer) {
         return;			// Not playing a level.
     }
@@ -1112,17 +1146,26 @@ bool KGrGame::inEditMode()
     return (editor != 0);	// Return true if the game-editor is active.
 }
 
-void KGrGame::toggleSoundsOnOff()
+void KGrGame::toggleSoundsOnOff (const int action)
 {
+    const char * setting = (action == PLAY_SOUNDS) ? "Sound" : "StepSounds";
     KConfigGroup gameGroup (KGlobal::config(), "KDEGame");
-    bool soundOnOff = gameGroup.readEntry ("Sound", false);
+    bool soundOnOff = gameGroup.readEntry (setting, false);
     soundOnOff = (! soundOnOff);
-    gameGroup.writeEntry ("Sound", soundOnOff);
+    gameGroup.writeEntry (setting, soundOnOff);
+    if (action == PLAY_SOUNDS) {
+        soundOn = soundOnOff;
+    }
+    else {
+        stepsOn = soundOnOff;
+    }
 #ifdef ENABLE_SOUND_SUPPORT
-    if (soundOnOff && (effects == 0)) {
-        loadSounds();
+    if (action == PLAY_SOUNDS) {
+        if (soundOn && (effects == 0)) {
+            loadSounds();	// Sounds were not loaded when the game started.
+        }
+        effects->setMuted (! soundOn);
     }
-    effects->setMuted (! soundOnOff);
 #endif
 }
 
@@ -1132,6 +1175,12 @@ void KGrGame::freeze (const bool userAction, const bool on_off)
     kDebug() << "PAUSE:" << type << on_off;
     kDebug() << "gameFrozen" << gameFrozen << "programFreeze" << programFreeze;
 
+#ifdef ENABLE_SOUND_SUPPORT
+    if (on_off && effects) {		// If pausing and sounds are loaded, cut
+        effects->stopAllSounds();	// off all sounds that are in progress.
+    }
+#endif
+
     if (! userAction) {
         // The program needs to freeze the game during a message, dialog, etc.
         if (on_off) {
@@ -1232,6 +1281,7 @@ void KGrGame::setPlayback (const bool onOff)
         emit setAvail  ("increase_speed",  enableDisable);
         emit setAvail  ("decrease_speed",  enableDisable);
     }
+    view->showReplayMessage (onOff);
     playback = onOff;
 }
 
@@ -1902,9 +1952,7 @@ bool KGrGame::initRecordingData (const Owner fileOwner, const QString & prefix,
                                  const int levelNo)
 {
     // Initialise the recording.
-    if (recording) {
-        delete recording;
-    }
+    delete recording;
     recording = new KGrRecording;
     recording->content.fill (0, 4000);
     recording->draws.fill   (0, 400);
@@ -2105,32 +2153,45 @@ bool KGrGame::loadRecording (const QString & dir, const QString & prefix,
 void KGrGame::loadSounds()
 {
 #ifdef ENABLE_SOUND_SUPPORT
-    effects = new KGrSoundBank(8);
+    const qreal volumes [NumSounds] = {0.6, 0.3, 0.3, 0.6, 0.6, 1.8, 1.0, 1.0, 1.0, 1.0};
+    #ifdef KGOLDRUNNER_USE_OPENAL
+    effects = new KGrSounds();
+    #else
+    effects = new KGrSoundBank (8);
+    #endif
     effects->setParent (this);		// Delete at end of KGrGame.
 
-    // StepSound and ClimbSound play better as .wav and the files are slightly
-    // shorter than the corresponding .ogg files.  All other sounds play well
-    // and play better concurrently as .ogg (with KDE 4.3 and Qt 4.5.1).
     fx[GoldSound]      = effects->loadSound (KStandardDirs::locate ("appdata",
-                         "themes/default/gold.ogg"));
+                         "themes/default/gold.wav"));
     fx[StepSound]      = effects->loadSound (KStandardDirs::locate ("appdata",
                          "themes/default/step.wav"));
     fx[ClimbSound]     = effects->loadSound (KStandardDirs::locate ("appdata",
                          "themes/default/climb.wav"));
     fx[FallSound]      = effects->loadSound (KStandardDirs::locate ("appdata",
-                         "themes/default/falling.ogg"));
+                         "themes/default/falling.wav"));
     fx[DigSound]       = effects->loadSound (KStandardDirs::locate ("appdata",
-                         "themes/default/dig.ogg"));
+                         "themes/default/dig.wav"));
     fx[LadderSound]    = effects->loadSound (KStandardDirs::locate ("appdata",
-                         "themes/default/ladder.ogg"));
+                         "themes/default/ladder.wav"));
     fx[CompletedSound] = effects->loadSound (KStandardDirs::locate ("appdata",
-                         "themes/default/completed.ogg"));
+                         "themes/default/completed.wav"));
     fx[DeathSound]     = effects->loadSound (KStandardDirs::locate ("appdata",
-                         "themes/default/death.ogg"));
+                         "themes/default/death.wav"));
     fx[GameOverSound]  = effects->loadSound (KStandardDirs::locate ("appdata",
-                         "themes/default/gameover.ogg"));
+                         "themes/default/gameover.wav"));
     fx[VictorySound]   = effects->loadSound (KStandardDirs::locate ("appdata",
-                         "themes/default/victory.ogg"));
+                         "themes/default/victory.wav"));
+    #ifdef KGOLDRUNNER_USE_OPENAL
+    // Gold and dig sounds are timed and are allowed to play for at least one
+    // second, so that rapid sequences of those sounds are heard as overlapping.
+    effects->setTimedSound (fx[GoldSound]);
+    effects->setTimedSound (fx[DigSound]);
+
+    // Adjust the relative volumes of sounds to improve the overall balance.
+    for (int i = 0; i < NumSounds; i++) {
+        effects->setVolume (fx [i], volumes [i]);
+    }
+    #endif
 #endif
 }
 
