@@ -41,6 +41,7 @@ KGrLevelPlayer::KGrLevelPlayer (QObject * parent, KRandomSequence * pRandomGen)
     randomGen        (pRandomGen),
     hero             (0),
     controlMode      (MOUSE),
+    holdKeyOption    (CLICK_KEY),
     nuggets          (0),
     playState        (NotReady),
     recording        (0),
@@ -54,7 +55,9 @@ KGrLevelPlayer::KGrLevelPlayer (QObject * parent, KRandomSequence * pRandomGen)
     digCycleCount    (40),	// Cycles while hole is fully open (default).
     digOpeningCycles (5),	// Cycles for brick-opening animation.
     digClosingCycles (4),	// Cycles for brick-closing animation.
-    digKillingTime   (2)	// Cycle at which enemy/hero gets killed.
+    digKillingTime   (2),	// Cycle at which enemy/hero gets killed.
+    dX               (0),	// X motion for KEYBOARD + HOLD_KEY option.
+    dY               (0)	// Y motion for KEYBOARD + HOLD_KEY option.
 {
     t.start(); // IDW
 
@@ -65,9 +68,8 @@ int KGrLevelPlayer::playerCount = 0;
 
 KGrLevelPlayer::~KGrLevelPlayer()
 {
-    while (! dugBricks.isEmpty()) {
-        delete dugBricks.takeFirst();
-    }
+    qDeleteAll(dugBricks);
+    dugBricks.clear(); //TODO: necessary?
     kDebug() << "LEVEL PLAYER BEING DELETED.";
     playerCount--;
 
@@ -94,8 +96,9 @@ if (recording) {
 
 }
 
-void KGrLevelPlayer::init (KGrCanvas * view, const int mode,
-                           KGrRecording * pRecording, const bool pPlayback,
+void KGrLevelPlayer::init (KGrCanvas * view,
+                           KGrRecording * pRecording,
+                           const bool pPlayback,
                            const bool gameFrozen)
 {
     // TODO - Remove?
@@ -112,7 +115,12 @@ void KGrLevelPlayer::init (KGrCanvas * view, const int mode,
     // Create the internal model of the level-layout.
     grid            = new KGrLevelGrid (this, recording);
 
-    controlMode     = mode;		// Set mouse/keyboard/laptop control.
+    // Set mouse/keyboard/laptop control and click/hold option for keyboard.
+    controlMode     = recording->controlMode;
+    holdKeyOption   = recording->keyOption;
+
+    dX              = 0;		// X motion for KEYBOARD + HOLD_KEY.
+    dY              = 0;		// Y motion for KEYBOARD + HOLD_KEY.
     levelWidth      = recording->width;
     levelHeight     = recording->height;
 
@@ -309,8 +317,6 @@ void KGrLevelPlayer::startDigging (Direction diggingDirection)
                     t.elapsed()}; // IDW test
         (* thisBrick)        = brick;
         dugBricks.append (thisBrick);
-        // kDebug() << "DIG" << thisBrick->id << thisBrick->countdown
-                 // << "time" << (t.elapsed() - thisBrick->startTime);
     }
 }
 
@@ -326,22 +332,16 @@ void KGrLevelPlayer::processDugBricks (const int scaledTime)
             dugBrick->cycleTimeLeft += digCycleTime;
             if (--dugBrick->countdown == digClosingCycles) {
                 // Start the brick-closing animation (non-repeating).
-                // kDebug() << "Brick" << dugBrick->digI << dugBrick->digJ <<
-                            // "count" << dugBrick->countdown;
                 emit startAnimation (dugBrick->id, false,
                                      dugBrick->digI, dugBrick->digJ,
                                      (digClosingCycles * digCycleTime),
                                      STAND, CLOSE_BRICK);
             }
             if (dugBrick->countdown == digKillingTime) {
-                // kDebug() << "Brick" << dugBrick->digI << dugBrick->digJ <<
-                            // "count" << dugBrick->countdown;
                 // Close the hole and maybe capture the hero or an enemy.
                 grid->changeCellAt (dugBrick->digI, dugBrick->digJ, BRICK);
             }
             if (dugBrick->countdown <= 0) {
-                // kDebug() << "DIG" << dugBrick->id << dugBrick->countdown
-                         // << "time" << (t.elapsed() - dugBrick->startTime);
                 // Dispose of the dug brick and remove it from the list.
                 emit deleteSprite (dugBrick->id);
                 delete dugBrick;
@@ -353,7 +353,6 @@ void KGrLevelPlayer::processDugBricks (const int scaledTime)
 
 void KGrLevelPlayer::prepareToPlay()
 {
-    kDebug() << "Set mouse to:" << targetI << targetJ;
     if ((controlMode == MOUSE) || (controlMode == LAPTOP)) {
         emit setMousePos (targetI, targetJ);
     }
@@ -432,7 +431,9 @@ void KGrLevelPlayer::doDig (int button)
     }
 }
 
-void KGrLevelPlayer::setDirectionByKey (Direction dirn)
+void KGrLevelPlayer::setDirectionByKey (const Direction dirn,
+                                        const bool pressed)
+                     
 {
     // Keystrokes control the hero.  KGrGame should avoid calling this during
     // playback, but better to be safe ...
@@ -447,6 +448,7 @@ void KGrLevelPlayer::setDirectionByKey (Direction dirn)
             T = 0;
         }
         if (controlMode == KEYBOARD) {
+// IDW What happens here if keyboard option is HOLD_KEY?  What *should* happen?
             newDirection = STAND;	// Stop a keyboard move when digging.
         }
         startDigging (dirn);
@@ -457,10 +459,15 @@ void KGrLevelPlayer::setDirectionByKey (Direction dirn)
             playState = Playing;
             T = 0;
         }
-        if (dirn != direction) {
-            // Start recording and acting on the new direction at the next tick.
+        // Start recording and acting on the new direction at the next tick.
+        if ((holdKeyOption == CLICK_KEY) && pressed && (dirn != direction)) {
             newDirection = dirn;
         }
+        else if (holdKeyOption == HOLD_KEY) {
+            int sign = pressed ? +1 : -1;
+            dX = dX + sign * movement [dirn][X];
+            dY = dY + sign * movement [dirn][Y];
+        }
     }
 }
 
@@ -470,43 +477,50 @@ Direction KGrLevelPlayer::getDirection (int heroI, int heroJ)
         int index = (playback) ? recIndex : recIndex - 2;
         dbe2 "T %04d recIndex %03d hero at [%02d, %02d] aiming at [%02d, %02d]\n",
              T, index, heroI, heroJ, targetI, targetJ);
+
         // If using a pointer device, calculate the hero's next direction,
         // starting from last pointer position and hero's current position.
 
-        int di = targetI - heroI;
-        int dj = targetJ - heroJ;
-
-        // kDebug() << "di" << di << "dj" << dj;
-
-        if ((dj > 0) && (grid->heroMoves (heroI, heroJ) & dFlag [DOWN])) {
-            // kDebug() << "Go down";
-            direction = DOWN;
-        }
-        else if ((dj < 0) && (grid->heroMoves (heroI, heroJ) & dFlag [UP])) {
-            // kDebug() << "Go up";
-            direction = UP;
-        }
-        else if (di > 0) {
-            // kDebug() << "Go right";
-            direction = RIGHT;
-        }
-        else if (di < 0) {
-            // kDebug() << "Go left";
-            direction = LEFT;
-        }
-        else {		// Note: di is zero, but dj is not necessarily zero.
-            // kDebug() << "Stand";
-            direction = STAND;
-        }
+        direction = setDirectionByDelta (targetI - heroI, targetJ - heroJ,
+                                         heroI, heroJ);
     }
-    else if (controlMode == KEYBOARD) {
-        dbe2 "T %04d recIndex %03d hero at [%02d, %02d] direction %d\n",
-             T, recIndex - 1, heroI, heroJ, direction);
+    else if ((controlMode == KEYBOARD) && (holdKeyOption == HOLD_KEY)) {
+
+        // If using the hold/release key option, resolve diagonal directions
+        // (if there are multi-key holds) into either horizontal or vertical.
+
+        direction = setDirectionByDelta (dX, dY, heroI, heroJ);
+        dbe2 "T %04d recIndex %03d delta [%02d, %02d] "
+             "hero at [%02d, %02d] direction %d\n",
+             T, recIndex - 1, dX, dY, heroI, heroJ, direction);
     }
 
     return direction;
 }
 
+Direction KGrLevelPlayer::setDirectionByDelta (const int di, const int dj,
+                                               const int heroI, const int heroJ)
+{
+    Direction dirn = STAND;
+
+    if ((dj > 0) && (grid->heroMoves (heroI, heroJ) & dFlag [DOWN])) {
+        dirn = DOWN;
+    }
+    else if ((dj < 0) && (grid->heroMoves (heroI, heroJ) & dFlag [UP])) {
+        dirn = UP;
+    }
+    else if (di > 0) {
+        dirn = RIGHT;
+    }
+    else if (di < 0) {
+        dirn = LEFT;
+    }
+    else {		// Note: di is zero, but dj is not necessarily zero.
+        dirn = STAND;
+    }
+    return dirn;
+}
+
 void KGrLevelPlayer::recordInitialWaitTime (const int ms)
 {
     // Allow a pause for viewing when playback starts.
@@ -741,11 +755,23 @@ void KGrLevelPlayer::tick (bool missed, int scaledTime)
         setTarget (i, j);
     }
     else if (controlMode == KEYBOARD) {
-        if (newDirection != direction) {
-            direction = newDirection;
+        if (holdKeyOption == CLICK_KEY) {
+            if (newDirection != direction) {
+                direction = newDirection;
+            }
+        }
+        // If keyboard with holdKey option, record one of nine directions.
+        else if (holdKeyOption == HOLD_KEY) {
+            const Direction d [9] = {UP_LEFT,   UP,    UP_RIGHT,
+                                     LEFT,      STAND, RIGHT,
+                                     DOWN_LEFT, DOWN,  DOWN_RIGHT};
+            direction = d [(3 * (dY + 1)) + (dX + 1)];
         }
         // Record the direction, but do not extend the initial wait-time.
-        if (direction != NO_DIRECTION) {
+        if ((direction != NO_DIRECTION) && (playState == Playing)) { // IDW
+// IDW Need a better condition here. (playState == Playing) was to stop
+// IDW the HOLD_KEY option recording a whole lot of STAND directions before
+// IDW the first key is pressed. (direction != NO_DIRECTION) is previous code.
             record (2, DIRECTION_CODE + direction);
         }
     }
@@ -961,6 +987,8 @@ bool KGrLevelPlayer::doRecordedMove()
                     dbe2 "T %04d recIndex %03d PLAY codes %d %d - KEY PRESS\n",
                          T, recIndex, code, recCount);
                     direction = ((Direction) (code));
+                    dX = movement [direction][X];
+                    dY = movement [direction][Y];
                 }
                 else {
                     dbe2 "T %04d recIndex %03d PLAY codes %d %d mode %d\n",
@@ -976,7 +1004,7 @@ bool KGrLevelPlayer::doRecordedMove()
         }
 
         // Replay a change of control-mode.
-        else if (code < ACTION_CODE) {
+        else if (code < KEY_OPT_CODE) {
             dbe2 "T %04d recIndex %03d PLAY control-mode code %d\n",
                  T, recIndex, code);
             setControlMode (code - MODE_CODE);
@@ -986,6 +1014,17 @@ bool KGrLevelPlayer::doRecordedMove()
             continue;
         }
 
+        // Replay a change of keyboard click/hold option.
+        else if (code < ACTION_CODE) {
+            dbe2 "T %04d recIndex %03d PLAY key-option code %d\n",
+                 T, recIndex, code);
+            setHoldKeyOption (code - KEY_OPT_CODE + CLICK_KEY);
+            recIndex++;
+            code = recording->content [recIndex];
+            recCount = 0;
+            continue;
+        }
+
         // Replay an action, such as KILL_HERO.
         else if (code < SPEED_CODE) {
             if (code == (ACTION_CODE + KILL_HERO)) {
@@ -1120,6 +1159,16 @@ void KGrLevelPlayer::setControlMode  (const int mode)
     }
 }
 
+void KGrLevelPlayer::setHoldKeyOption  (const int option)
+{
+    holdKeyOption = option;
+
+    if (! playback) {
+        // Record the change of keyboard click/hold option.
+        record (1, KEY_OPT_CODE + option - CLICK_KEY);
+    }
+}
+
 void KGrLevelPlayer::setTimeScale  (const int timeScale)
 {
     timer->setScale ((float) (timeScale * 0.1));
