@@ -27,13 +27,18 @@ lateralWidget::lateralWidget(QWidget *parent) : QWidget(parent)
 	m_rolls = new QLabel(this);
 	m_dices = new dicesWidget(this);
 	m_buttons = new QStackedWidget(this);
-	m_rollButton = new QPushButton(KIcon("roll"), i18n("&Roll"), this);
+	m_rollButton = new QPushButton(KIcon( QLatin1String( "roll") ), i18n("&Roll" ), this);
 	
 	KAction *dummyAction = KStandardGameAction::gameNew(0, 0, 0);
 	m_newGameButton = new QPushButton(dummyAction->icon(), dummyAction->text(), this);
 	delete dummyAction;
 
-	m_newGameButton -> setEnabled(false);
+	if (!m_demoMode) m_newGameButton -> setEnabled(false);
+	else
+	{
+		m_newGameButton->setEnabled(true);
+		m_dices->setEnabled(false);
+	}
 
 	lay -> addWidget(m_rolls, 0, Qt::AlignHCenter);
 	lay -> addWidget(m_dices);
@@ -43,6 +48,7 @@ lateralWidget::lateralWidget(QWidget *parent) : QWidget(parent)
 	lay -> addStretch(1);
 
 	connect(m_rollButton, SIGNAL(clicked(bool)), this, SLOT(roll()));
+	connect(m_rollButton, SIGNAL(clicked(bool)), this, SLOT(unhighlightAllDice()));
 	connect(m_newGameButton, SIGNAL(clicked(bool)), this, SLOT(newGame()));
 	
 	nextTurn();
@@ -53,7 +59,7 @@ void lateralWidget::nextTurn()
 	m_roll = 1;
 	updateRollLabel();
 	m_dices -> rollAll();
-	m_buttons -> setCurrentWidget(m_rollButton);
+	if (!m_demoMode) m_buttons -> setCurrentWidget(m_rollButton);
 }
 
 void lateralWidget::setEnabled(bool enabled)
@@ -64,6 +70,20 @@ void lateralWidget::setEnabled(bool enabled)
 	m_dices -> setEnabled(enabled);
 }
 
+void lateralWidget::enableDemoMode()
+{
+	m_buttons -> setCurrentWidget(m_newGameButton);
+	m_newGameButton->setEnabled(true);
+	m_demoMode = true;
+}
+
+void lateralWidget::disableDemoMode()
+{
+	m_buttons -> setCurrentWidget(m_rollButton);
+	m_newGameButton -> setEnabled(false);
+	m_demoMode = false;
+}
+
 void lateralWidget::endGame()
 {
 	m_newGameButton -> setEnabled(true);
@@ -80,6 +100,16 @@ void lateralWidget::selectDice(int dice, bool selected)
 	m_dices -> selectDice(dice, selected);
 }
 
+void lateralWidget::highlightDice(int dice, bool selected)
+{
+	m_dices -> highlightDice(dice, selected);
+}
+
+void lateralWidget::unhighlightAllDice()
+{
+	for (int i = 0; i < 5; ++i) m_dices -> highlightDice(i, false);
+}
+
 int lateralWidget::getRolls() const
 {
 	return m_roll;
@@ -157,19 +187,20 @@ void lateralWidget::roll()
 	{
 		m_roll++;
 		updateRollLabel();
+		emit rolled();
 	}
 }
 
 void lateralWidget::newGame()
 {
-	m_newGameButton -> setEnabled(false);
+	disableDemoMode();
 	emit newGameClicked();
 }
 
 void lateralWidget::updateRollLabel()
 {
 	m_rolls -> setText(i18n("Roll %1 of 3",m_roll));
-	setEnabled(m_roll != 3);
+	if (!m_demoMode) setEnabled(m_roll != 3);
 }
 
 #include "lateralwidget.moc"
