@@ -69,13 +69,10 @@ public:
     virtual void relayoutScene();
     void updateWonItem();
 
-
     virtual void addPile( KCardPile * pile );
     virtual void removePile( KCardPile * pile );
     QList<PatPile*> patPiles() const;
 
-    virtual void moveCardsToPile( QList<KCard*> cards, KCardPile * pile, int duration );
-
     void setAutoDropEnabled( bool enabled );
     bool autoDropEnabled() const;
 
@@ -153,6 +150,8 @@ protected:
     virtual bool checkRemove( const PatPile * pile, const QList<KCard*> & cards ) const;
     virtual bool checkPrefering( const PatPile * pile, const QList<KCard*> & oldCards, const QList<KCard*> & newCards ) const;
 
+    virtual void cardsMoved( const QList<KCard*> & cards, KCardPile * oldPile, KCardPile * newPile );
+
     virtual void mouseDoubleClickEvent( QGraphicsSceneMouseEvent * mouseEvent );
     virtual void mousePressEvent( QGraphicsSceneMouseEvent * mouseEvent );
     virtual void mouseReleaseEvent( QGraphicsSceneMouseEvent * mouseEvent );
@@ -167,12 +166,12 @@ protected:
     virtual QList<MoveHint> getHints();
 
     // reimplement these to store and load game-specific information in the state structure
-    virtual QString getGameState() { return QString(); }
-    virtual void setGameState( const QString & ) {}
+    virtual QString getGameState() const;
+    virtual void setGameState( const QString & state );
 
     // reimplement these to store and load game-specific options in the saved game file
-    virtual QString getGameOptions() const { return QString(); }
-    virtual void setGameOptions( const QString & ) {}
+    virtual QString getGameOptions() const;
+    virtual void setGameOptions( const QString & options );
 
     void addCardForDeal( KCardPile * pile, KCard * card, bool faceUp, QPointF startPos );
     void startDealAnimation();
@@ -183,6 +182,13 @@ protected:
     void setDeckContents( int copies = 1,
                           const QList<KCardDeck::Suit> & suits = KCardDeck::standardSuits() );
 
+    void multiStepMove( const QList<KCard*> & cards,
+                        KCardPile * pile,
+                        const QList<KCardPile*> & freePiles,
+                        const QList<KCardPile*> & freeCells,
+                        int duration  );
+
+
 protected slots:
     void takeState();
     virtual void animationDone();
@@ -214,6 +220,12 @@ private:
 
     int speedUpTime( int delay ) const;
 
+    void multiStepSubMove( QList<KCard*> cards,
+                           KCardPile * pile,
+                           QList<KCardPile*> freePiles,
+                           const QList<KCardPile*> & freeCells );
+    void continueMultiStepMove();
+
     class DealerScenePrivate;
     DealerScenePrivate *d;
 };
