@@ -51,15 +51,27 @@
 #define DEBUG_LAYOUT 0
 
 
-const int cardMoveDuration = 230;
-
-
-inline QRectF multRectSize( const QRectF & rect, const QSize & size )
+namespace
 {
-    return QRectF( rect.x() * size.width(), 
-                   rect.y() * size.height(),
-                   rect.width() * size.width(),
-                   rect.height() * size.height() );
+    const int cardMoveDuration = 230;
+
+
+    void setItemHighlight( QGraphicsItem * item, bool highlight )
+    {
+        KCard * card = qgraphicsitem_cast<KCard*>( item );
+        if ( card )
+        {
+            card->setHighlighted( highlight );
+            return;
+        }
+
+        KCardPile * pile = qgraphicsitem_cast<KCardPile*>( item );
+        if ( pile )
+        {
+            pile->setHighlighted( highlight );
+            return;
+        }
+    }
 }
 
 
@@ -68,7 +80,8 @@ class KCardScenePrivate : public QObject
 public:
     KCardScenePrivate( KCardScene * p );
 
-    int calculateDuration( QPointF pos1, QPointF pos2, qreal velocity ) const;
+    KCardPile * bestDestinationPileUnderCards();
+    void sendCardsToPile( KCardPile * pile, QList<KCard*> cards, qreal rate, bool isSpeed, bool flip );
     void changeFocus( int pileChange, int cardChange );
     void updateKeyboardFocus();
 
@@ -101,17 +114,156 @@ KCardScenePrivate::KCardScenePrivate( KCardScene * p )
   : QObject( p ),
     q( p )
 {
+  dragStarted = false;
 }
 
 
-int KCardScenePrivate::calculateDuration( QPointF pos1, QPointF pos2, qreal velocity ) const
+KCardPile * KCardScenePrivate::bestDestinationPileUnderCards()
 {
-    QPointF delta = pos2 - pos1;
-    qreal distance = sqrt( delta.x() * delta.x() + delta.y() * delta.y() );
-    qreal cardUnit = ( deck->cardWidth() + deck->cardHeight() ) / 2.0;
-    qreal unitDistance = distance / cardUnit;
+    QSet<KCardPile*> targets;
+    foreach ( QGraphicsItem * item, q->collidingItems( cardsBeingDragged.first(), Qt::IntersectsItemBoundingRect ) )
+    {
+        KCardPile * p = qgraphicsitem_cast<KCardPile*>( item );
+        if ( !p )
+        {
+            KCard * c = qgraphicsitem_cast<KCard*>( item );
+            if ( c )
+                p = c->pile();
+        }
+        if ( p )
+            targets << p;
+    }
 
-    return 1000 * unitDistance / velocity;
+    KCardPile * bestTarget = 0;
+    qreal bestArea = 1;
+
+    foreach ( KCardPile * p, targets )
+    {
+        if ( p != cardsBeingDragged.first()->pile() && q->allowedToAdd( p, cardsBeingDragged ) )
+        {
+            QRectF targetRect = p->sceneBoundingRect();
+            foreach ( KCard *c, p->cards() )
+                targetRect |= c->sceneBoundingRect();
+
+            QRectF intersection = targetRect & cardsBeingDragged.first()->sceneBoundingRect();
+            qreal area = intersection.width() * intersection.height();
+            if ( area > bestArea )
+            {
+                bestTarget = p;
+                bestArea = area;
+            }
+        }
+    }
+
+    return bestTarget;
+}
+
+
+void KCardScenePrivate::sendCardsToPile( KCardPile * pile, QList<KCard*> newCards, qreal rate, bool isSpeed, bool flip )
+{
+    if ( pile->isEmpty() && newCards.isEmpty() )
+        return;
+
+    int oldCardCount = pile->count();
+
+    for ( int i = 0; i < newCards.size(); ++i )
+    {
+        // If we're flipping the cards, we have to add them to the pile in
+        // reverse order.
+        KCard * c = newCards[ flip ? newCards.size() - 1 - i : i ];
+
+        // The layout of the card within the pile may depend on whether it is
+        // face up or down. Therefore, we must flip the card, add it to the
+        // pile, calculate its position, flip it back, then start the animation
+        // which will flip it one more time.
+        if ( flip )
+            c->setFaceUp( !c->isFaceUp() );
+
+        pile->add( c );
+    }
+
+    const QSize cardSize = deck->cardSize();
+    const qreal cardUnit = (deck->cardWidth() + deck->cardHeight()) / 2.0;
+    const QList<KCard*> cards = pile->cards();
+    const QList<QPointF> positions = pile->cardPositions();
+
+    qreal minX = 0;
+    qreal maxX = 0;
+    qreal minY = 0;
+    qreal maxY = 0;
+    foreach ( const QPointF & pos, positions )
+    {
+        minX = qMin( minX, pos.x() );
+        maxX = qMax( maxX, pos.x() );
+        minY = qMin( minY, pos.y() );
+        maxY = qMax( maxY, pos.y() );
+    }
+
+    QPointF absLayoutPos = pile->layoutPos();
+    if ( absLayoutPos.x() < 0 )
+        absLayoutPos.rx() += contentSize.width() / cardSize.width() - 1;
+    if ( absLayoutPos.y() < 0 )
+        absLayoutPos.ry() += contentSize.height() / cardSize.height() - 1;
+
+    QRectF available = pileAreas.value( pile, QRectF() );
+    qreal availableTop = absLayoutPos.y() - available.top();
+    qreal availableBottom = available.bottom() - (absLayoutPos.y() + 1);
+    qreal availableLeft = absLayoutPos.x() - available.left();
+    qreal availableRight = available.right() - (absLayoutPos.x() + 1);
+
+    qreal scaleTop = (minY >= 0) ? 1 : qMin<qreal>( availableTop / -minY, 1 );
+    qreal scaleBottom = (maxY <= 0) ? 1 : qMin<qreal>( availableBottom / maxY, 1 );
+    qreal scaleY = qMin( scaleTop, scaleBottom );
+
+    qreal scaleLeft = (minX >= 0) ? 1 : qMin<qreal>( availableLeft / -minX, 1 );
+    qreal scaleRight = (maxX <= 0) ? 1 : qMin<qreal>( availableRight / maxX, 1 );
+    qreal scaleX = qMin( scaleLeft, scaleRight );
+
+    QList<QPointF> realPositions;
+    QList<qreal> distances;
+    qreal maxDistance = 0;
+    for ( int i = 0; i < cards.size(); ++i )
+    {
+        QPointF pos( pile->x() + positions[i].x() * scaleX * cardSize.width(),
+                     pile->y() + positions[i].y() * scaleY * cardSize.height() );
+        realPositions << pos;
+
+        qreal distance = 0;
+        if ( isSpeed && i >= oldCardCount )
+        {
+            QPointF delta = pos - cards[i]->pos();
+            distance = sqrt( delta.x() * delta.x() + delta.y() * delta.y() ) / cardUnit;
+            if ( distance > maxDistance )
+                maxDistance = distance;
+        }
+        distances << distance;
+    }
+
+    qreal z = pile->zValue();
+    int layoutDuration = isSpeed ? qMin<int>( cardMoveDuration, maxDistance / rate * 1000 ) : rate;
+
+    for ( int i = 0; i < cards.size(); ++i )
+    {
+        bool isNewCard = i >= oldCardCount;
+
+        int duration = (isNewCard && isSpeed) ? distances[i] / rate * 1000 : layoutDuration;
+
+        // Honour the pile's autoTurnTop property.
+        bool face = cards[i]->isFaceUp() || (cards[i] == pile->topCard() && pile->autoTurnTop());
+
+        if ( isNewCard && flip )
+        {
+            // The card will be flipped as part of the animation, so we return
+            // it to its original face up/down position before starting the
+            // animation.
+            cards[i]->setFaceUp( !cards[i]->isFaceUp() );
+        }
+
+        // Each card has a Z value 1 greater than the card below it.
+        ++z;
+
+        cards[i]->animate( realPositions[i], z, 0, face, isNewCard, duration );
+    }
 }
 
 
@@ -190,7 +342,7 @@ void KCardScenePrivate::changeFocus( int pileChange, int cardChange )
 
 void KCardScenePrivate::updateKeyboardFocus()
 {
-    q->setItemHighlight( keyboardFocusItem.data(), false );
+    setItemHighlight( keyboardFocusItem.data(), false );
 
     if ( !keyboardMode )
     {
@@ -220,14 +372,14 @@ void KCardScenePrivate::updateKeyboardFocus()
     }
     else if ( keyboardCardIndex >= pile->count() || hint == KCardPile::ForceFocusTop )
     {
-        keyboardFocusItem = pile->top();
+        keyboardFocusItem = pile->topCard();
     }
     else
     {
         keyboardFocusItem = pile->at( keyboardCardIndex );
     }
 
-    q->setItemHighlight( keyboardFocusItem.data(), true );
+    setItemHighlight( keyboardFocusItem.data(), true );
 
     QPointF delta = keyboardFocusItem.data()->pos() - startOfDrag;
     startOfDrag = keyboardFocusItem.data()->pos();
@@ -260,8 +412,6 @@ KCardScene::~KCardScene()
         delete p;
     }
     d->piles.clear();
-
-    Q_ASSERT( items().isEmpty() );
 }
 
 
@@ -323,12 +473,6 @@ QList<KCardPile*> KCardScene::piles() const
 }
 
 
-QRectF KCardScene::spaceAllottedToPile( const KCardPile * pile ) const
-{
-    return d->pileAreas.value( pile, QRectF() );
-}
-
-
 void KCardScene::setSceneAlignment( KCardScene::SceneAlignment alignment )
 {
     if ( alignment != d->alignment )
@@ -486,11 +630,13 @@ void KCardScene::relayoutScene()
     d->contentSize = QSizeF( contentWidth, contentHeight );
     setSceneRect( -xOffset, -yOffset, width(), height() );
 
-    relayoutPiles( 0 );
+    recalculatePileLayouts();
+    foreach ( KCardPile * p, piles() )
+        updatePileLayout( p, 0 );
 }
 
 
-void KCardScene::relayoutPiles( int duration )
+void KCardScene::recalculatePileLayouts()
 {
     if ( !d->sizeHasBeenSet || !d->deck )
         return;
@@ -602,9 +748,6 @@ void KCardScene::relayoutPiles( int duration )
             }
         }
     }
-
-    foreach ( KCardPile * p, piles() )
-        p->layoutCards( duration );
 }
 
 
@@ -633,41 +776,15 @@ QList<QGraphicsItem*> KCardScene::highlightedItems() const
 }
 
 
-void KCardScene::setItemHighlight( QGraphicsItem * item, bool highlight )
-{
-    KCard * card = qgraphicsitem_cast<KCard*>( item );
-    if ( card )
-    {
-        card->setHighlighted( highlight );
-        return;
-    }
-
-    KCardPile * pile = qgraphicsitem_cast<KCardPile*>( item );
-    if ( pile )
-    {
-        pile->setHighlighted( highlight );
-        return;
-    }
-}
-
-
-void KCardScene::moveCardsToPile( QList<KCard*> cards, KCardPile * pile, int duration )
+void KCardScene::moveCardsToPile( const QList<KCard*> & cards, KCardPile * pile, int duration )
 {
     if ( cards.isEmpty() )
         return;
 
     KCardPile * source = cards.first()->pile();
-
-    foreach ( KCard * c, cards )
-    {
-        Q_ASSERT( c->pile() == source );
-        pile->add( c );
-        c->raise();
-    }
-
-    source->layoutCards( duration );
-
-    pile->layoutCards( duration );
+    d->sendCardsToPile( pile, cards, duration, false, false );
+    d->sendCardsToPile( source, QList<KCard*>(), duration, false, false );
+    cardsMoved( cards, source, pile );
 }
 
 
@@ -677,58 +794,33 @@ void KCardScene::moveCardToPile( KCard * card, KCardPile * pile, int duration )
 }
 
 
-void KCardScene::moveCardToPileAtSpeed( KCard * card, KCardPile * pile, qreal velocity )
+void KCardScene::moveCardsToPileAtSpeed( const QList<KCard*> & cards, KCardPile * pile, qreal velocity )
 {
-    QPointF origPos = card->pos();
+    if ( cards.isEmpty() )
+        return;
 
-    QPointF estimatedDestPos = pile->isEmpty() ? pile->pos() : pile->top()->pos();
-    moveCardToPile( card, pile, d->calculateDuration( origPos, estimatedDestPos, velocity ) );
-
-    card->completeAnimation();
-    QPointF destPos = card->pos();
-    card->setPos( origPos );
-
-    int duration = d->calculateDuration( origPos, destPos, velocity );
-    card->animate( destPos, card->zValue(), 0, card->isFaceUp(), true, duration );
+    KCardPile * source = cards.first()->pile();
+    d->sendCardsToPile( pile, cards, velocity, true, false );
+    d->sendCardsToPile( source, QList<KCard*>(), cardMoveDuration, false, false );
+    cardsMoved( cards, source, pile );
 }
 
 
-void KCardScene::flipCardsToPile( QList<KCard*> cards, KCardPile * pile, int duration )
+void KCardScene::moveCardToPileAtSpeed( KCard * card, KCardPile * pile, qreal velocity )
 {
-    QList<KCard*> revCards;
-    QList<bool> origFaces;
-    QList<QPointF> origPositions;
-    QList<qreal> origZValues;
+    moveCardsToPileAtSpeed( QList<KCard*>() << card, pile, velocity );
+}
 
-    for ( int i = cards.size() - 1; i >= 0; --i )
-    {
-        KCard * c = cards.at( i );
-        revCards << c;
-        origFaces << c->isFaceUp();
-        origZValues << c->zValue();
-        origPositions << c->pos();
-    }
 
-    moveCardsToPile( revCards, pile, duration );
+void KCardScene::flipCardsToPile( const QList<KCard*> & cards, KCardPile * pile, int duration )
+{
+    if ( cards.isEmpty() )
+        return;
 
-    for ( int i = 0; i < revCards.size(); ++i )
-    {
-        KCard * c = revCards.at( i );
-
-        c->completeAnimation();
-        c->setFaceUp( origFaces.at( i ) );
-        QPointF destPos = c->pos();
-        c->setPos( origPositions.at( i ) );
-        qreal destZValue = c->zValue();
-
-        // This is a bit of a hack. It means we preserve the z ordering of face
-        // up cards, but feel free to mess about with face down ones. This may
-        // need to be smarter in the future.
-        if ( c->isFaceUp() )
-            c->setZValue( origZValues.at( i ) );
-
-        c->animate( destPos, destZValue, 0, !c->isFaceUp(), true, duration );
-    }
+    KCardPile * source = cards.first()->pile();
+    d->sendCardsToPile( pile, cards, duration, false, true );
+    d->sendCardsToPile( source, QList<KCard*>(), duration, false, false );
+    cardsMoved( cards, source, pile );
 }
 
 
@@ -738,22 +830,21 @@ void KCardScene::flipCardToPile( KCard * card, KCardPile * pile, int duration )
 }
 
 
+void KCardScene::flipCardsToPileAtSpeed( const QList<KCard*> & cards, KCardPile * pile, qreal velocity )
+{
+    if ( cards.isEmpty() )
+        return;
+
+    KCardPile * source = cards.first()->pile();
+    d->sendCardsToPile( pile, cards, velocity, true, true );
+    d->sendCardsToPile( source, QList<KCard*>(), cardMoveDuration, false, false );
+    cardsMoved( cards, source, pile );
+}
+
+
 void KCardScene::flipCardToPileAtSpeed( KCard * card, KCardPile * pile, qreal velocity )
 {
-    QPointF origPos = card->pos();
-    bool origFaceUp = card->isFaceUp();
-
-    QPointF estimatedDestPos = pile->isEmpty() ? pile->pos() : pile->top()->pos();
-    card->setFaceUp( !origFaceUp );
-    moveCardToPile( card, pile, d->calculateDuration( origPos, estimatedDestPos, velocity ) );
-
-    card->completeAnimation();
-    QPointF destPos = card->pos();
-    card->setPos( origPos );
-    card->setFaceUp( origFaceUp );
-
-    int duration = d->calculateDuration( origPos, destPos, velocity );
-    card->animate( destPos, card->zValue(), 0, !origFaceUp, true, duration );
+    flipCardsToPileAtSpeed( QList<KCard*>() << card, pile, velocity );
 }
 
 
@@ -829,11 +920,11 @@ void KCardScene::keyboardFocusSelect()
     }
     else
     {
-        KCardPile * destination = targetPile();
+        KCardPile * destination = d->bestDestinationPileUnderCards();
         if ( destination )
-            moveCardsToPile( d->cardsBeingDragged, destination, cardMoveDuration );
+            cardsDroppedOnPile( d->cardsBeingDragged, destination );
         else
-            d->cardsBeingDragged.first()->pile()->layoutCards( cardMoveDuration );
+            updatePileLayout( d->cardsBeingDragged.first()->pile(), cardMoveDuration );
 
         QGraphicsItem * toFocus = d->cardsBeingDragged.first();
         d->cardsBeingDragged.clear();
@@ -876,7 +967,7 @@ void KCardScene::setKeyboardModeActive( bool keyboardMode )
     else if ( d->keyboardMode && !keyboardMode )
     {
         if ( !d->cardsBeingDragged.isEmpty() )
-            d->cardsBeingDragged.first()->pile()->layoutCards( cardMoveDuration );
+            updatePileLayout( d->cardsBeingDragged.first()->pile(), cardMoveDuration );
         d->cardsBeingDragged.clear();
 
         d->keyboardMode = false;
@@ -891,6 +982,12 @@ bool KCardScene::isKeyboardModeActive() const
 }
 
 
+void KCardScene::updatePileLayout( KCardPile * pile, int duration )
+{
+    d->sendCardsToPile( pile, QList<KCard*>(), duration, false, false );
+}
+
+
 bool KCardScene::allowedToAdd( const KCardPile * pile, const QList<KCard*> & cards ) const
 {
     Q_UNUSED( pile )
@@ -907,44 +1004,17 @@ bool KCardScene::allowedToRemove( const KCardPile * pile, const KCard * card ) c
 }
 
 
-KCardPile * KCardScene::targetPile()
+void KCardScene::cardsDroppedOnPile( const QList<KCard*> & cards, KCardPile * pile )
 {
-    QSet<KCardPile*> targets;
-    foreach ( QGraphicsItem * item, collidingItems( d->cardsBeingDragged.first() ) )
-    {
-        KCardPile * p = qgraphicsitem_cast<KCardPile*>( item );
-        if ( !p )
-        {
-            KCard * c = qgraphicsitem_cast<KCard*>( item );
-            if ( c )
-                p = c->pile();
-        }
-        if ( p )
-            targets << p;
-    }
+    moveCardsToPile( cards, pile, cardMoveDuration );
+}
 
-    KCardPile * bestTarget = 0;
-    qreal bestArea = 1;
 
-    foreach ( KCardPile * p, targets )
-    {
-        if ( p != d->cardsBeingDragged.first()->pile() && allowedToAdd( p, d->cardsBeingDragged ) )
-        {
-            QRectF targetRect = p->sceneBoundingRect();
-            foreach ( KCard *c, p->cards() )
-                targetRect |= c->sceneBoundingRect();
-
-            QRectF intersection = targetRect & d->cardsBeingDragged.first()->sceneBoundingRect();
-            qreal area = intersection.width() * intersection.height();
-            if ( area > bestArea )
-            {
-                bestTarget = p;
-                bestArea = area;
-            }
-        }
-    }
-
-    return bestTarget;
+void KCardScene::cardsMoved( const QList<KCard*> & cards, KCardPile * oldPile, KCardPile * newPile )
+{
+    Q_UNUSED( cards );
+    Q_UNUSED( oldPile );
+    Q_UNUSED( newPile );
 }
 
 
@@ -1018,13 +1088,13 @@ void KCardScene::mouseMoveEvent( QGraphicsSceneMouseEvent * e )
             d->startOfDrag = e->scenePos();
 
             QList<QGraphicsItem*> toHighlight;
-            KCardPile * dropPile = targetPile();
+            KCardPile * dropPile = d->bestDestinationPileUnderCards();
             if ( dropPile )
             {
                 if ( dropPile->isEmpty() )
                     toHighlight << dropPile;
                 else
-                    toHighlight << dropPile->top();
+                    toHighlight << dropPile->topCard();
             }
 
             setHighlightedItems( toHighlight );
@@ -1045,7 +1115,7 @@ void KCardScene::mouseReleaseEvent( QGraphicsSceneMouseEvent * e )
 
     if ( e->button() == Qt::LeftButton && !d->dragStarted && !d->cardsBeingDragged.isEmpty() )
     {
-        d->cardsBeingDragged.first()->pile()->layoutCards( cardMoveDuration );
+        updatePileLayout( d->cardsBeingDragged.first()->pile(), cardMoveDuration );
         d->cardsBeingDragged.clear();
     }
 
@@ -1053,11 +1123,11 @@ void KCardScene::mouseReleaseEvent( QGraphicsSceneMouseEvent * e )
     {
         e->accept();
 
-        KCardPile * destination = targetPile();
+        KCardPile * destination = d->bestDestinationPileUnderCards();
         if ( destination )
-            moveCardsToPile( d->cardsBeingDragged, destination, cardMoveDuration );
+            cardsDroppedOnPile( d->cardsBeingDragged, destination );
         else
-            d->cardsBeingDragged.first()->pile()->layoutCards( cardMoveDuration );
+            updatePileLayout( d->cardsBeingDragged.first()->pile(), cardMoveDuration );
         d->cardsBeingDragged.clear();
         d->dragStarted = false;
         clearHighlightedItems();
@@ -1107,7 +1177,7 @@ void KCardScene::mouseDoubleClickEvent( QGraphicsSceneMouseEvent * e )
 
     if ( !d->cardsBeingDragged.isEmpty() )
     {
-        d->cardsBeingDragged.first()->pile()->layoutCards( cardMoveDuration );
+        updatePileLayout( d->cardsBeingDragged.first()->pile(), cardMoveDuration );
         d->cardsBeingDragged.clear();
     }
 
@@ -1140,7 +1210,9 @@ void KCardScene::wheelEvent( QGraphicsSceneWheelEvent * e )
         qreal scaleFactor = pow( 2, e->delta() / qreal(10 * 120) );
         int newWidth = d->deck->cardWidth() * scaleFactor;
         d->deck->setCardWidth( newWidth );
-        relayoutPiles();
+        recalculatePileLayouts();
+        foreach ( KCardPile * p, piles() )
+            updatePileLayout( p, 0 );
     }
     else
     {
@@ -1166,7 +1238,7 @@ void KCardScene::drawForeground ( QPainter * painter, const QRectF & rect )
         if ( !p->isVisible() )
             continue;
 
-        QRectF availableRect = multRectSize( spaceAllottedToPile( p ), d->deck->cardSize() );
+        QRectF availableRect = multRectSize( d->pileAreas.value( p, QRectF() ), d->deck->cardSize() );
 
         QRectF reservedRect( -p->leftPadding(), -p->topPadding(), 1 + p->rightPadding(), 1 + p->bottomPadding() );
         reservedRect = multRectSize( reservedRect, d->deck->cardSize() );
