@@ -40,8 +40,6 @@
 #include "pileutils.h"
 #include "patsolve/yukonsolver.h"
 
-#include "Shuffle"
-
 #include <KLocale>
 
 
@@ -50,7 +48,7 @@ void Yukon::initialize()
     const qreal dist_x = 1.11;
     const qreal dist_y = 1.11;
 
-    static_cast<KStandardCardDeck*>( deck() )->setDeckContents();
+    setDeckContents();
 
     for ( int i = 0; i < 4; ++i )
     {
@@ -67,7 +65,8 @@ void Yukon::initialize()
         store[i]->setPileRole(PatPile::Tableau);
         store[i]->setLayoutPos(dist_x*i, 0);
         store[i]->setAutoTurnTop(true);
-        store[i]->setReservedSpace( 0, 0, 1, 3 * dist_y + 1.0 );
+        store[i]->setBottomPadding( 3 * dist_y );
+        store[i]->setHeightPolicy( KCardPile::GrowDown );
         store[i]->setKeyboardSelectHint( KCardPile::FreeFocus );
         store[i]->setKeyboardDropHint( KCardPile::AutoFocusTop );
     }
@@ -82,10 +81,10 @@ bool Yukon::checkAdd(const PatPile * pile, const QList<KCard*> & oldCards, const
     if (pile->pileRole() == PatPile::Tableau)
     {
         if (oldCards.isEmpty())
-            return getRank( newCards.first() ) == KStandardCardDeck::King;
+            return newCards.first()->rank() == KCardDeck::King;
         else
-            return getRank( newCards.first() ) == getRank( oldCards.last() ) - 1
-                   && getIsRed( newCards.first() ) != getIsRed( oldCards.last() );
+            return newCards.first()->rank() == oldCards.last()->rank() - 1
+                   && newCards.first()->color() != oldCards.last()->color();
     }
     else
     {
@@ -98,9 +97,9 @@ bool Yukon::checkRemove(const PatPile * pile, const QList<KCard*> & cards) const
     return pile->pileRole() == PatPile::Tableau && cards.first()->isFaceUp();
 }
 
-void Yukon::restart()
+void Yukon::restart( const QList<KCard*> & cards )
 {
-    QList<KCard*> cards = shuffled( deck()->cards(), gameNumber() );
+    QList<KCard*> cardList = cards;
 
     for ( int round = 0; round < 11; ++round )
     {
@@ -110,7 +109,7 @@ void Yukon::restart()
             {
                 QPointF initPos = store[j]->pos();
                 initPos.ry() += ((7 - j / 3.0) + round) * deck()->cardHeight();
-                addCardForDeal( store[j], cards.takeLast(), (round >= j || j == 0), initPos );
+                addCardForDeal( store[j], cardList.takeLast(), (round >= j || j == 0), initPos );
             }
         }
     }
